// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "Stage.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ACTIVATEAPP()
	ON_WM_PALETTECHANGED()
	ON_WM_MOVE()
	ON_WM_SETFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{
	m_pScene = NULL;
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Create the 3D window
	if (!m_wnd3d.Create(this, IDC_3DWND)) {
		return -1;
	}

	// enable mouse selection of objects
	m_wnd3d.EnableMouseSelection(OnSelChange, this);
	
	// Create an initial scene
	m_pScene = new C3dScene;
	if (!m_pScene->Create()) return -1;
	m_wnd3d.SetScene(m_pScene);

	// Set up the lighting
    C3dPtLight light;
    light.Create(0.8, 0.8, 0.8);
    m_pScene->AddChild(&light);
    light.SetDirection(1, -1, 1);
	light.SetPosition(-3, 6, -10);
    m_pScene->SetAmbientLight(0.4, 0.4, 0.4);
	m_pScene->SetBackground(0.4, 1, 1); // sky blue

	// set the camera position and direction
    m_pScene->SetCameraPosition(C3dVector(0, 0, -10));
    m_pScene->SetCameraDirection(C3dVector(0, 0, 1));

	// Add the globe
	m_pScene->AddChild(&m_globe);

	// make the selection pin a child of the globe
	m_globe.AddChild(&m_pin);

	// Create the shadow for the globe
	m_globe.CreateShadow(&light, C3dVector(0, 0, 5), C3dVector(0, 0, -1));

	// Set the orientation of the axis
	C3dVector vAxis(0.5, 0.8, 0);
	m_globe.SetDirection(vAxis);

	// make it rotate
	m_globe.SetRotation(vAxis.x, vAxis.y, vAxis.z, 0.01);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// Restore any saved settings
	RestoreSettings();

	// Restore the window size and position
	cs.x = m_rcWnd.left;
	cs.y = m_rcWnd.top;
	cs.cx = m_rcWnd.Width();
	cs.cy = m_rcWnd.Height();

	return CFrameWnd::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

void CMainFrame::OnDestroy() 
{
	CFrameWnd::OnDestroy();

	// Save the current settings
	SaveSettings();
	
	// Delete any scene we might have
	m_wnd3d.SetScene(NULL);
	if (m_pScene) {
		delete m_pScene;
		m_pScene = NULL;
	}
}

/////////////////////////////////////////////////////////////////////////////
// window layout handler

void CMainFrame::RecalcLayout(BOOL bNotify) 
{
    // Rearrange the control bars and fit the 3D window in the middle
	// Let the frame rearrange the control bars
	CFrameWnd::RecalcLayout(bNotify);

	// Find the space that's left over
	CRect rc;
	RepositionBars(0,
				   0xFFFF,
				   IDC_3DWND,
				   CWnd::reposQuery,
				   &rc);
	if (IsWindow(m_wnd3d.GetSafeHwnd())) {
		m_wnd3d.MoveWindow(&rc, FALSE);
	}
}


///////////////////////////////////////////////////////////////
// Windows messages we need to handle

void CMainFrame::OnActivateApp(BOOL bActive, HTASK hTask) 
{
	CFrameWnd::OnActivateApp(bActive, hTask);
	
	// Tell the 3D window about the new state
	m_wnd3d.SendMessage(WM_ACTIVATEAPP,
						(WPARAM)bActive,
						(LPARAM)hTask);
}

void CMainFrame::OnPaletteChanged(CWnd* pFocusWnd) 
{
	// Let the 3D window know
	m_wnd3d.SendMessage(WM_PALETTECHANGED,
					   pFocusWnd ? (WPARAM)pFocusWnd->GetSafeHwnd() : 0);
}

void CMainFrame::OnMove(int x, int y) 
{
	CFrameWnd::OnMove(x, y);
	
	// Let the 3D window know the frame has moved
	m_wnd3d.SendMessage(WM_MOVE, 0, MAKELPARAM(0, 0));
}

void CMainFrame::OnSetFocus(CWnd* pOldWnd) 
{
	CFrameWnd::OnSetFocus(pOldWnd);
	
	// Give the focus to the 3D window so it can process
	// keystrokes for the key input device
	m_wnd3d.SetFocus();
}

//////////////////////////////////////////////////////////////////
// Save and Restore Settings

void CMainFrame::SaveSettings()
{
	CWinApp* pa = AfxGetApp();
	CRect rc;
	GetWindowRect(&rc);
	pa->WriteProfileInt("Settings\\Window","Top", rc.top);
	pa->WriteProfileInt("Settings\\Window","Left", rc.left);
	pa->WriteProfileInt("Settings\\Window","Width", rc.Width());
	pa->WriteProfileInt("Settings\\Window","Height", rc.Height());
}

void CMainFrame::RestoreSettings()
{
	CWinApp* pa = AfxGetApp();
	m_rcWnd.top = pa->GetProfileInt("Settings\\Window","Top", 50);
	m_rcWnd.left = pa->GetProfileInt("Settings\\Window","Left", 50);
	m_rcWnd.right = m_rcWnd.left + pa->GetProfileInt("Settings\\Window","Width", 300); 
	m_rcWnd.bottom = m_rcWnd.top + pa->GetProfileInt("Settings\\Window","Height", 400);
}

//////////////////////////////////////////////////////////////////////////
// object selection

// callback notification function
// called by m_wnd3d when the object selection changes
// static
void CMainFrame::OnSelChange(C3dShape* pShape, CPoint pt, void* pArg)
{
	// Get a pointer to the class
	CMainFrame* pThis = (CMainFrame*) pArg;
	ASSERT(pThis);
	ASSERT(pThis->IsKindOf(RUNTIME_CLASS(CMainFrame)));

	int iFace;
	C3dVector vHit;
	if (pShape) {

		// Find which face was hit
		C3dViewport* pViewport = pThis->m_wnd3d.GetStage()->GetViewport();
		pShape->HitTest(pt, pViewport, &iFace, &vHit);

	}

	// Make the selected shape the current one
	pThis->ShowHit(pShape, iFace, vHit);
}

void CMainFrame::ShowHit(C3dShape* pShape, int iFace, C3dVector vHit)
{
	if (!pShape) return; // ignore non-globe hits

	// put the pin in the globe
	// Get the face normal
	C3dVector vn = pShape->GetFaceNormal(iFace);

	// Point to the hit point with the pin
	m_pin.SetPosition(vHit);
	m_pin.SetDirection(vn);

	// Compute the latitude and longitude
	double lat, lon;
	m_globe.GetLatLon(vHit, lat, lon);

	char buf[256];
	sprintf(buf,
			"X: %3.1f, Y: %3.1f, Z: %3.1f, Lat: %4.1f, Lon: %5.1f\n",
			vHit.x, vHit.y, vHit.z,
			lat, lon);
	TRACE(buf);


}


