// globe.cpp

#include "stdafx.h"
#include "Stage.h"
#include <math.h>

// country description
typedef struct _COUNTRY	{
	POINT* pCoords;
	int nCoords;
	char* szName;
	COLORREF clr;
	UINT uiSound;
} COUNTRY;

// country polygon lists
POINT cNorth [] = {
	{0, 90}, {358, 90}, {359, 59}, {0, 50}
};
int nNorth = sizeof(cNorth) / sizeof(POINT);

POINT cTrop	[] = {
	{120, -30}, {30, -30}, {30, 35}, {60, 35,}, {60, 0}, {120, 0}
};
int nTrop = sizeof(cTrop) / sizeof(POINT);

POINT cSouth [] = {
	{0, -90}, {359, -90}, {359, -50}, {0, -50}
};
int nSouth = sizeof(cSouth) / sizeof(POINT);

POINT cMtn [] = {
	{150, 0}, {180, 30}, {210, 0}, {180, -30}
};
int nMtn = sizeof(cMtn) / sizeof(POINT);

POINT cIsl [] = {
	{300, 0}, {300, 30}, {330, 30}, {330, 0}
};
int nIsl = sizeof(cIsl) / sizeof(POINT);

// country descriptions
COUNTRY North = {
	cNorth,
	nNorth,
	"North Pole",
	RGB(255, 255, 255),
	IDS_NORTH
};

COUNTRY Trop = {
	cTrop,
	nTrop,
	"Tropicana",
	RGB(255, 0, 0),
	IDS_TROPIC
};

COUNTRY South = {
	cSouth,
	nSouth,
	"South Pole",
	RGB(255, 255, 255),
	IDS_SOUTH
};

COUNTRY Mtn = {
	cMtn,
	nMtn,
	"Mountain Land",
	RGB(0, 255, 0),
	IDS_MTN
};

COUNTRY Isl = {
	cIsl,
	nIsl,
	"The Island",
	RGB(255, 255, 0),
	IDS_ISLAND
};

// country list
COUNTRY* Countries [] = {
	&North,
	&Trop,
	&Mtn,
	&Isl,
	&South,
	NULL
};

static COUNTRY* GetCountry(double lat, double lon)
{
	if (lon < 0) lon = 360 + lon;
	CPoint pt((int) lon,
			  (int) lat);

	COUNTRY** ppc = Countries;
	while (*ppc) {
		if (_3dPointInPolygon((*ppc)->pCoords, (*ppc)->nCoords, pt)) {
			break;
		}
		ppc++;
	}

	return *ppc;
}

CCrust::CCrust()
{
	// Create a BITMAPINFO structure to describe the bitmap
	int iWidth = 512;
	int iHeight = 256;
	int iSize = sizeof(BITMAPINFO) + 256 * sizeof(RGBQUAD);
	BITMAPINFO* pBMI = (BITMAPINFO*) new BYTE[iSize];
	memset(pBMI, 0, iSize);
	pBMI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	pBMI->bmiHeader.biWidth = iWidth; 
	pBMI->bmiHeader.biHeight = iHeight;
	pBMI->bmiHeader.biPlanes = 1;
	pBMI->bmiHeader.biBitCount = 8;
	pBMI->bmiHeader.biClrUsed = 256;

	HDC hdcScreen = ::GetDC(NULL);

	// create the color 
	PALETTEENTRY pe[256];
	::GetSystemPaletteEntries(hdcScreen, 0, 256, pe);
	for (int i = 0; i < 256; i++) {
		pBMI->bmiColors[i].rgbRed = pe[i].peRed;
		pBMI->bmiColors[i].rgbGreen = pe[i].peGreen;
		pBMI->bmiColors[i].rgbBlue = pe[i].peBlue;
	}

	// Create a DIB section
	BYTE* pBits = NULL;
	HBITMAP hBmp = ::CreateDIBSection(hdcScreen,
									  pBMI,
									  DIB_RGB_COLORS,
									  (VOID**)&pBits,
									  NULL,
									  0);
	ASSERT(hBmp);
	ASSERT(pBits);

	// Create a memory DC to play with
	HDC hdcMem = ::CreateCompatibleDC(hdcScreen);
	HBITMAP hbmOld = (HBITMAP)::SelectObject(hdcMem, hBmp);

	// Set the mapping mode so we can work in lon/lat scale
	::SetMapMode(hdcMem, MM_ANISOTROPIC);
	::SetWindowExtEx(hdcMem, 360, 180, NULL);
	::SetWindowOrgEx(hdcMem, 0, 0, NULL);
	::SetViewportExtEx(hdcMem, iWidth, -iHeight, NULL);
	::SetViewportOrgEx(hdcMem, 0, iHeight/2, NULL);

	// Draw the world crust picture into the memory DC
	HBRUSH hbrSea = ::CreateSolidBrush(RGB(0, 0, 255));
	HBRUSH hbrOld = (HBRUSH)::SelectObject(hdcMem, hbrSea);
	HPEN hpenAxes = ::CreatePen(PS_SOLID, 1, RGB(255, 255, 255));
	HPEN hpenOld = (HPEN)::SelectObject(hdcMem, hpenAxes);

	::Rectangle(hdcMem, 0, 0, 180, 90);
	::Rectangle(hdcMem, 0, -90, 180, 0);
	::Rectangle(hdcMem, 180, 0, 360, 90);
	::Rectangle(hdcMem, 180, -90, 360, 0);
	::SelectObject(hdcMem, hbrOld);

	// Draw all the countries
	COUNTRY** ppc = Countries;
	while (*ppc) {

		// Create the land color brush
		HBRUSH hbrLand = ::CreateSolidBrush((*ppc)->clr);
		HBRUSH hbr = (HBRUSH)::SelectObject(hdcMem, hbrLand);
		::Polygon(hdcMem, (*ppc)->pCoords, (*ppc)->nCoords);
		ppc++;
		::SelectObject(hdcMem, hbr);
	}

	// Clean up
 	::SelectObject(hdcMem, hpenOld);
	::SelectObject(hdcMem, hbmOld);
	::DeleteDC(hdcMem);
   	::ReleaseDC(NULL, hdcScreen);

	// Create the image object
    RGBQUAD* pRGB = (RGBQUAD*)(((BYTE*)pBMI)+sizeof(BITMAPINFOHEADER));
	C3dImage::_Create((BITMAPINFOHEADER*)pBMI, pRGB, pBits);

	// Create the texture map from the image
	C3dTexture::Create();

	// Done with the bitmap info header now
	delete pBMI;
}

CGlobe::CGlobe()
{
	m_r = 2.5;

	// Create a world
	CreateSphere(m_r);

	// Add a mountain
	C3dShape cone;
	cone.CreateCone(0, -m_r, 0, 0.2, FALSE,
					0, -m_r * 1.1, 0, 0, FALSE);
	cone.SetColor(0.5, 0.5, 0.5);
	AddChild(&cone);

	// Create the texture map of the crust
	CCrust* pTex = new CCrust;
	m_ImgList.Append(pTex);

	// Attach the texture to the shape
	SetTexture(pTex);

	// Create a spherical wrap
	C3dWrap wrap;
    wrap.Create(D3DRMWRAP_SPHERE,
				NULL,
				0, 0, 0, // origin
				0, 0, 1, // direction
				0, 1, 0, // up
				0, 0,	 // texture origin
				1, 1);	 // texture scale

	// apply the wrap to the shapes
	wrap.Apply(this);
}

void CGlobe::GetLatLon(C3dVector& v, double& lat, double &lon)
{
	lat = asin(v.z / m_r) *  57.29577951308; // degrees
	lon = atan2(v.x, v.y) *  57.29577951308; // degrees

	// figure out which country it is
	COUNTRY* pCountry = ::GetCountry(lat, lon);
	if (pCountry) {

		// say what country it is
		PlaySound(MAKEINTRESOURCE(pCountry->uiSound),
			      AfxGetResourceHandle(),
			      SND_RESOURCE | SND_ASYNC);
	}
}


CPin::CPin()
{
	double l = 1;
	CreateRod(0, 0, 0, 0, 0, l, 0.05);
	SetColor(0, 0.5, 1);
	C3dShape ball;
	ball.CreateSphere(0.2);
	ball.SetColor(1, 0, 0);
	AddChild(&ball);
	ball.SetPosition(0, 0, l);
}


